@@ -1,13 +1,14 @@
 /* movemail foo bar -- move file foo to file bar,
    locking file foo the way /bin/mail respects.
-   Copyright (C) 1986, 92, 93, 94, 96, 1999 Free Software Foundation, Inc.
+   Copyright (C) 1986, 1992, 1993, 1994, 1996, 1999, 2001, 2002, 2003, 2004,
+                 2005, 2006, 2007, 2008, 2009, 2010, 2011  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
-GNU Emacs is free software; you can redistribute it and/or modify
+GNU Emacs is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +16,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
+
 
 /* Important notice: defining MAIL_USE_FLOCK or MAIL_USE_LOCKF *will
    cause loss of mail* if you do it on a system that does not normally
@@ -39,11 +39,11 @@ Boston, MA 02111-1307, USA.  */
  * "po:username".  This will cause movemail to open a connection to
  * a pop server running on $MAILHOST (environment variable).  Movemail
  * must be setuid to root in order to work with POP.
- * 
+ *
  * New module: popmail.c
  * Modified routines:
  *	main - added code within #ifdef MAIL_USE_POP; added setuid (getuid ())
- *		after POP code. 
+ *		after POP code.
  * New routines in movemail.c:
  *	get_errmsg - return pointer to system error message
  *
@@ -54,13 +54,13 @@ Boston, MA 02111-1307, USA.  */
  *
  */
 
-#define NO_SHORTNAMES   /* Tell config not to load remap.h */
 #include <config.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/file.h>
 #include <stdio.h>
 #include <errno.h>
+#include <time.h>
 
 #include <getopt.h>
 #ifdef HAVE_UNISTD_H
@@ -112,7 +112,7 @@ Boston, MA 02111-1307, USA.  */
 #define R_OK 4
 #endif
 
-#if defined (XENIX) || defined (WINDOWSNT)
+#ifdef WINDOWSNT
 #include <sys/locking.h>
 #endif
 
@@ -129,7 +129,8 @@ extern int lk_open (), lk_close ();
 #endif
 
 #if !defined (MAIL_USE_SYSTEM_LOCK) && !defined (MAIL_USE_MMDF) && \
-	defined (HAVE_LIBMAIL) && defined (HAVE_MAILLOCK_H)
+	(defined (HAVE_LIBMAIL) || defined (HAVE_LIBLOCKFILE)) && \
+        defined (HAVE_MAILLOCK_H)
 #include <maillock.h>
 /* We can't use maillock unless we know what directory system mail
    files appear in. */
@@ -173,7 +174,7 @@ main (argc, argv)
   char *inname, *outname;
   int indesc, outdesc;
   int nread;
-  WAITTYPE status;
+  int status;
   int c, preserve_mail = 0;
 
 #ifndef MAIL_USE_SYSTEM_LOCK
@@ -196,6 +197,9 @@ main (argc, argv)
 # define ARGSTR "p"
 #endif /* MAIL_USE_POP */
 
+  uid_t real_gid = getgid();
+  uid_t priv_gid = getegid();
+
 #ifdef WINDOWSNT
   /* Ensure all file i/o is in binary mode. */
   _fmode = _O_BINARY;
@@ -215,7 +219,7 @@ main (argc, argv)
 	preserve_mail++;
 	break;
       default:
-	exit(1);
+	exit (EXIT_FAILURE);
       }
     }
 
@@ -227,14 +231,13 @@ main (argc, argv)
 #endif
       )
     {
-      fprintf (stderr, "Usage: movemail [-p] inbox destfile%s\n",
 #ifdef MAIL_USE_POP
-	       " [POP-password]"
+      fprintf (stderr, "Usage: movemail [-p] [-r] inbox destfile%s\n",
+	       " [POP-password]");
 #else
-	       ""
+      fprintf (stderr, "Usage: movemail [-p] inbox destfile%s\n", "");
 #endif
-	       );
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   inname = argv[optind];
@@ -245,26 +248,7 @@ main (argc, argv)
 #endif
 
   if (*outname == 0)
-    fatal ("Destination file name is empty", 0);
-
-  /* Check access to output file.  */
-  if (access (outname, F_OK) == 0 && access (outname, W_OK) != 0)
-    pfatal_with_name (outname);
-
-  /* Also check that outname's directory is writable to the real uid.  */
-  {
-    char *buf = (char *) xmalloc (strlen (outname) + 1);
-    char *p;
-    strcpy (buf, outname);
-    p = buf + strlen (buf);
-    while (p > buf && !IS_DIRECTORY_SEP (p[-1]))
-      *--p = 0;
-    if (p == buf)
-      *p++ = '.';
-    if (access (buf, W_OK) != 0)
-      pfatal_with_name (buf);
-    free (buf);
-  }
+    fatal ("Destination file name is empty", 0, 0);
 
 #ifdef MAIL_USE_POP
   if (!strncmp (inname, "po:", 3))
@@ -277,15 +261,12 @@ main (argc, argv)
       exit (status);
     }
 
-  setuid (getuid ());
+  if (setuid (getuid ()) < 0)
+    fatal ("Failed to drop privileges", 0, 0);
+
 #endif /* MAIL_USE_POP */
 
 #ifndef DISABLE_DIRECT_ACCESS
-
-  /* Check access to input file.  */
-  if (access (inname, R_OK | W_OK) != 0)
-    pfatal_with_name (inname);
-
 #ifndef MAIL_USE_MMDF
 #ifndef MAIL_USE_SYSTEM_LOCK
 #ifdef MAIL_USE_MAILLOCK
@@ -342,6 +323,13 @@ main (argc, argv)
 	  close (desc);
 
 	  tem = link (tempname, lockname);
+
+#ifdef EPERM
+	  if (tem < 0 && errno == EPERM)
+	    fatal ("Unable to create hard link between %s and %s",
+		   tempname, lockname);
+#endif
+
 	  unlink (tempname);
 	  if (tem >= 0)
 	    break;
@@ -369,10 +357,11 @@ main (argc, argv)
       int lockcount = 0;
       int status = 0;
 #if defined (MAIL_USE_MAILLOCK) && defined (HAVE_TOUCHLOCK)
-      long touched_lock, now;
+      time_t touched_lock, now;
 #endif
 
-      setuid (getuid ());
+      if (setuid (getuid ()) < 0 || setregid (-1, real_gid) < 0)
+	fatal ("Failed to drop privileges", 0, 0);
 
 #ifndef MAIL_USE_MMDF
 #ifdef MAIL_USE_SYSTEM_LOCK
@@ -387,17 +376,20 @@ main (argc, argv)
       if (indesc < 0)
 	pfatal_with_name (inname);
 
-#if defined (BSD_SYSTEM) || defined (XENIX)
+#ifdef BSD_SYSTEM
       /* In case movemail is setuid to root, make sure the user can
 	 read the output file.  */
       /* This is desirable for all systems
 	 but I don't want to assume all have the umask system call */
       umask (umask (0) & 0333);
-#endif /* BSD_SYSTEM || XENIX */
+#endif /* BSD_SYSTEM */
       outdesc = open (outname, O_WRONLY | O_CREAT | O_EXCL, 0666);
       if (outdesc < 0)
 	pfatal_with_name (outname);
 
+      if (setregid (-1, priv_gid) < 0)
+	fatal ("Failed to regain privileges", 0, 0);
+
       /* This label exists so we can retry locking
 	 after a delay, if it got EAGAIN or EBUSY.  */
     retry_lock:
@@ -421,15 +413,11 @@ main (argc, argv)
 #ifdef MAIL_USE_LOCKF
 	  status = lockf (indesc, F_LOCK, 0);
 #else /* not MAIL_USE_LOCKF */
-#ifdef XENIX
-	  status = locking (indesc, LK_RLCK, 0L);
-#else
 #ifdef WINDOWSNT
 	  status = locking (indesc, LK_RLCK, -1L);
 #else
 	  status = flock (indesc, LOCK_EX);
 #endif
-#endif
 #endif /* not MAIL_USE_LOCKF */
 #endif /* MAIL_USE_SYSTEM_LOCK */
 	}
@@ -458,13 +446,15 @@ main (argc, argv)
 
 	  pfatal_with_name (inname);
 	}
-  
+
       {
 	char buf[1024];
 
 	while (1)
 	  {
 	    nread = read (indesc, buf, sizeof buf);
+	    if (nread < 0)
+	      pfatal_with_name (inname);
 	    if (nread != write (outdesc, buf, nread))
 	      {
 		int saved_errno = errno;
@@ -493,6 +483,10 @@ main (argc, argv)
 	pfatal_and_delete (outname);
 #endif
 
+      /* Prevent symlink attacks truncating other users' mailboxes */
+      if (setregid (-1, real_gid) < 0)
+	fatal ("Failed to drop privileges", 0, 0);
+
       /* Check to make sure no errors before we zap the inbox.  */
       if (close (outdesc) != 0)
 	pfatal_and_delete (outname);
@@ -500,13 +494,7 @@ main (argc, argv)
 #ifdef MAIL_USE_SYSTEM_LOCK
       if (! preserve_mail)
 	{
-#if defined (STRIDE) || defined (XENIX)
-	  /* Stride, xenix have file locking, but no ftruncate.
-	     This mess will do. */
-	  close (open (inname, O_CREAT | O_TRUNC | O_RDWR, 0666));
-#else
 	  ftruncate (indesc, 0L);
-#endif /* STRIDE or XENIX */
 	}
 #endif /* MAIL_USE_SYSTEM_LOCK */
 
@@ -530,18 +518,22 @@ main (argc, argv)
 	}
 #endif /* not MAIL_USE_SYSTEM_LOCK */
 
+      /* End of mailbox truncation */
+      if (setregid (-1, priv_gid) < 0)
+	fatal ("Failed to regain privileges", 0, 0);
+
 #ifdef MAIL_USE_MAILLOCK
       /* This has to occur in the child, i.e., in the process that
          acquired the lock! */
       if (spool_name)
 	mailunlock ();
 #endif
-      exit (0);
+      exit (EXIT_SUCCESS);
     }
 
   wait (&status);
   if (!WIFEXITED (status))
-    exit (1);
+    exit (EXIT_FAILURE);
   else if (WRETCODE (status) != 0)
     exit (WRETCODE (status));
 
@@ -554,7 +546,7 @@ main (argc, argv)
 
 #endif /* ! DISABLE_DIRECT_ACCESS */
 
-  return 0;
+  return EXIT_SUCCESS;
 }
 
 #ifdef MAIL_USE_MAILLOCK
@@ -601,13 +593,13 @@ mail_spool_name (inname)
 /* Print error message and exit.  */
 
 void
-fatal (s1, s2)
-     char *s1, *s2;
+fatal (s1, s2, s3)
+     char *s1, *s2, *s3;
 {
   if (delete_lockname)
     unlink (delete_lockname);
-  error (s1, s2, 0);
-  exit (1);
+  error (s1, s2, s3);
+  exit (EXIT_FAILURE);
 }
 
 /* Print error message.  `s1' is printf control string, `s2' and `s3'
@@ -631,17 +623,16 @@ void
 pfatal_with_name (name)
      char *name;
 {
-  char *s = concat ("", strerror (errno), " for %s");
-  fatal (s, name);
+  fatal ("%s for %s", strerror (errno), name);
 }
 
 void
 pfatal_and_delete (name)
      char *name;
 {
-  char *s = concat ("", strerror (errno), " for %s");
+  char *s = strerror (errno);
   unlink (name);
-  fatal (s, name);
+  fatal ("%s for %s", s, name);
 }
 
 /* Return a newly-allocated string whose contents concatenate those of s1, s2, s3.  */
@@ -669,7 +660,7 @@ xmalloc (size)
 {
   long *result = (long *) malloc (size);
   if (!result)
-    fatal ("virtual memory exhausted", 0);
+    fatal ("virtual memory exhausted", 0, 0);
   return result;
 }
 
@@ -686,6 +677,7 @@ xmalloc (size)
 #include <winsock.h>
 #endif
 #include <pwd.h>
+#include <string.h>
 
 #define NOTOK (-1)
 #define OK 0
@@ -700,7 +692,7 @@ char Errmsg[200];		/* POP errors, at least, can exceed
 				   the original length of 80.  */
 
 /*
- * The full legal syntax for a POP mailbox specification for movemail
+ * The full valid syntax for a POP mailbox specification for movemail
  * is "po:username:hostname".  The ":hostname" is optional; if it is
  * omitted, the MAILHOST environment variable will be consulted.  Note
  * that by the time popmail() is called the "po:" has been stripped
@@ -709,6 +701,8 @@ char Errmsg[200];		/* POP errors, at least, can exceed
  * If the mailbox is in the form "po:username:hostname", then it is
  * modified by this function -- the second colon is replaced by a
  * null.
+ *
+ * Return a value suitable for passing to `exit'.
  */
 
 int
@@ -736,19 +730,19 @@ popmail (mailbox, outfile, preserve, password, reverse_order)
   if (! server)
     {
       error ("Error connecting to POP server: %s", pop_error, 0);
-      return (1);
+      return EXIT_FAILURE;
     }
 
   if (pop_stat (server, &nmsgs, &nbytes))
     {
       error ("Error getting message count from POP server: %s", pop_error, 0);
-      return (1);
+      return EXIT_FAILURE;
     }
 
   if (!nmsgs)
     {
       pop_close (server);
-      return (0);
+      return EXIT_SUCCESS;
     }
 
   mbfi = open (outfile, O_WRONLY | O_CREAT | O_EXCL, 0666);
@@ -756,7 +750,7 @@ popmail (mailbox, outfile, preserve, password, reverse_order)
     {
       pop_close (server);
       error ("Error in open: %s, %s", strerror (errno), outfile);
-      return (1);
+      return EXIT_FAILURE;
     }
   fchown (mbfi, getuid (), -1);
 
@@ -766,7 +760,7 @@ popmail (mailbox, outfile, preserve, password, reverse_order)
       error ("Error in fdopen: %s", strerror (errno), 0);
       close (mbfi);
       unlink (outfile);
-      return (1);
+      return EXIT_FAILURE;
     }
 
   if (reverse_order)
@@ -789,7 +783,7 @@ popmail (mailbox, outfile, preserve, password, reverse_order)
 	{
 	  error ("%s", Errmsg, 0);
 	  close (mbfi);
-	  return (1);
+	  return EXIT_FAILURE;
 	}
       mbx_delimit_end (mbf);
       fflush (mbf);
@@ -798,7 +792,7 @@ popmail (mailbox, outfile, preserve, password, reverse_order)
 	  error ("Error in fflush: %s", strerror (errno), 0);
 	  pop_close (server);
 	  close (mbfi);
-	  return (1);
+	  return EXIT_FAILURE;
 	}
     }
 
@@ -812,14 +806,14 @@ popmail (mailbox, outfile, preserve, password, reverse_order)
   if (fsync (mbfi) < 0)
     {
       error ("Error in fsync: %s", strerror (errno), 0);
-      return (1);
+      return EXIT_FAILURE;
     }
 #endif
 
   if (close (mbfi) == -1)
     {
       error ("Error in close: %s", strerror (errno), 0);
-      return (1);
+      return EXIT_FAILURE;
     }
 
   if (! preserve)
@@ -829,22 +823,23 @@ popmail (mailbox, outfile, preserve, password, reverse_order)
 	  {
 	    error ("Error from POP server: %s", pop_error, 0);
 	    pop_close (server);
-	    return (1);
+	    return EXIT_FAILURE;
 	  }
       }
 
   if (pop_quit (server))
     {
       error ("Error from POP server: %s", pop_error, 0);
-      return (1);
+      return EXIT_FAILURE;
     }
-    
-  return (0);
+
+  return EXIT_SUCCESS;
 }
 
 int
 pop_retr (server, msgno, arg)
      popserver server;
+     int msgno;
      FILE *arg;
 {
   extern char *strerror ();
@@ -912,7 +907,7 @@ mbx_write (line, len, mbf)
       line++;
       len--;
     }
-  if (fwrite (line, 1, len, mbf) != len) 
+  if (fwrite (line, 1, len, mbf) != len)
     return (NOTOK);
   if (fputc (0x0a, mbf) == EOF)
     return (NOTOK);
@@ -923,7 +918,16 @@ int
 mbx_delimit_begin (mbf)
      FILE *mbf;
 {
-  if (fputs ("\f\n0, unseen,,\n", mbf) == EOF)
+  time_t now;
+  struct tm *ltime;
+  char fromline[40] = "From movemail ";
+
+  now = time (NULL);
+  ltime = localtime (&now);
+
+  strcat (fromline, asctime (ltime));
+
+  if (fputs (fromline, mbf) == EOF)
     return (NOTOK);
   return (OK);
 }
@@ -932,7 +936,7 @@ int
 mbx_delimit_end (mbf)
      FILE *mbf;
 {
-  if (putc ('\037', mbf) == EOF)
+  if (putc ('\n', mbf) == EOF)
     return (NOTOK);
   return (OK);
 }
@@ -953,3 +957,8 @@ strerror (errnum)
 }
 
 #endif /* ! HAVE_STRERROR */
+
+/* arch-tag: 1c323112-41fe-4fe5-8de9-494de631f73f
+   (do not change this comment) */
+
+/* movemail.c ends here */
