@@ -1,14 +1,15 @@
 /* pop.c: client routines for talking to a POP3-protocol post-office server
    Copyright (C) 1991, 1993, 1996, 1997, 1999, 2001, 2002, 2003, 2004,
-                 2005, 2006, 2007, 2008  Free Software Foundation, Inc.
-   Written by Jonathan Kamens, jik@security.ov.com.
+                 2005, 2006, 2007, 2008, 2009, 2010, 2011  Free Software Foundation, Inc.
+
+Author: Jonathan Kamens <jik@security.ov.com>
 
 This file is part of GNU Emacs.
 
-GNU Emacs is free software; you can redistribute it and/or modify
+GNU Emacs is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3, or (at your option)
-any later version.
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,12 +17,10 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-Boston, MA 02110-1301, USA.  */
+along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
+
 
 #ifdef HAVE_CONFIG_H
-#define NO_SHORTNAMES	/* Tell config not to load remap.h */
 #include <config.h>
 #else
 #define MAIL_USE_POP
@@ -999,8 +998,7 @@ pop_quit (server)
       close (server->file);
     }
 
-  if (server->buffer)
-    free (server->buffer);
+  free (server->buffer);
   free ((char *) server);
 
   return (ret);
@@ -1029,12 +1027,19 @@ socket_connection (host, flags)
      char *host;
      int flags;
 {
+#ifdef HAVE_GETADDRINFO
+  struct addrinfo *res, *it;
+  struct addrinfo hints;
+  int ret;
+#else /* !HAVE_GETADDRINFO */
   struct hostent *hostent;
+#endif
   struct servent *servent;
   struct sockaddr_in addr;
   char found_port = 0;
   char *service;
   int sock;
+  char *realhost;
 #ifdef KERBEROS
 #ifdef KERBEROS5
   krb5_error_code rem;
@@ -1050,11 +1055,11 @@ socket_connection (host, flags)
   CREDENTIALS cred;
   Key_schedule schedule;
   int rem;
-  char *realhost;
 #endif /* KERBEROS5 */
 #endif /* KERBEROS */
 
   int try_count = 0;
+  int connect_ok;
 
 #ifdef WINDOWSNT
   {
@@ -1116,6 +1121,46 @@ socket_connection (host, flags)
 
     }
 
+#ifdef HAVE_GETADDRINFO
+  memset (&hints, 0, sizeof(hints));
+  hints.ai_socktype = SOCK_STREAM;
+  hints.ai_flags = AI_CANONNAME;
+  hints.ai_family = AF_INET;
+  do
+    {
+      ret = getaddrinfo (host, service, &hints, &res);
+      try_count++;
+      if (ret != 0 && (ret != EAI_AGAIN || try_count == 5))
+	{
+	  strcpy (pop_error, "Could not determine POP server's address");
+	  return (-1);
+	}
+    } while (ret != 0);
+
+  if (ret == 0)
+    {
+      it = res;
+      while (it)
+        {
+          if (it->ai_addrlen == sizeof (addr))
+            {
+              struct sockaddr_in *in_a = (struct sockaddr_in *) it->ai_addr;
+              bcopy (&in_a->sin_addr, (char *) &addr.sin_addr,
+                     sizeof (addr.sin_addr));
+              if (! connect (sock, (struct sockaddr *) &addr, sizeof (addr)))
+                break;
+            }
+          it = it->ai_next;
+        }
+      connect_ok = it != NULL;
+      if (connect_ok)
+        {
+          realhost = alloca (strlen (it->ai_canonname) + 1);
+          strcpy (realhost, it->ai_canonname);
+        }
+      freeaddrinfo (res);
+    }
+#else /* !HAVE_GETADDRINFO */
   do
     {
       hostent = gethostbyname (host);
@@ -1135,10 +1180,18 @@ socket_connection (host, flags)
 	break;
       hostent->h_addr_list++;
     }
+  connect_ok = *hostent->h_addr_list != NULL;
+  if (! connect_ok)
+    {
+      realhost = alloca (strlen (hostent->h_name) + 1);
+      strcpy (realhost, hostent->h_name);
+    }
+
+#endif /* !HAVE_GETADDRINFO */
 
 #define CONNECT_ERROR "Could not connect to POP server: "
 
-  if (! *hostent->h_addr_list)
+  if (! connect_ok)
     {
       CLOSESOCKET (sock);
       strcpy (pop_error, CONNECT_ERROR);
@@ -1149,6 +1202,7 @@ socket_connection (host, flags)
     }
 
 #ifdef KERBEROS
+
 #define KRB_ERROR "Kerberos error connecting to POP server: "
   if (! (flags & POP_NO_KERBEROS))
     {
@@ -1176,7 +1230,7 @@ socket_connection (host, flags)
       if (rem = krb5_cc_get_principal (kcontext, ccdef, &client))
 	goto krb5error;
 
-      for (cp = hostent->h_name; *cp; cp++)
+      for (cp = realhost; *cp; cp++)
 	{
 	  if (isupper (*cp))
 	    {
@@ -1184,7 +1238,7 @@ socket_connection (host, flags)
 	    }
 	}
 
-      if (rem = krb5_sname_to_principal (kcontext, hostent->h_name,
+      if (rem = krb5_sname_to_principal (kcontext, realhost,
 					 POP_SERVICE, FALSE, &server))
 	goto krb5error;
 
@@ -1235,7 +1289,6 @@ socket_connection (host, flags)
 	}
 #else  /* ! KERBEROS5 */
       ticket = (KTEXT) malloc (sizeof (KTEXT_ST));
-      realhost = strdup (hostent->h_name);
       rem = krb_sendauth (0L, sock, ticket, "pop", realhost,
 			  (char *) krb_realmofhost (realhost),
 			  (unsigned long) 0, &msg_data, &cred, schedule,
@@ -1243,7 +1296,6 @@ socket_connection (host, flags)
 			  (struct sockaddr_in *) 0,
 			  "KPOPV0.1");
       free ((char *) ticket);
-      free (realhost);
       if (rem != KSUCCESS)
 	{
 	  strcpy (pop_error, KRB_ERROR);
