@@ -2,7 +2,7 @@
    standard output a file of texinfo input containing the doc strings.
 
 Copyright (C) 1989, 1992, 1994, 1996, 1999, 2000, 2001, 2002, 2003,
-              2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+              2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
