@@ -1,9 +1,12 @@
 ;;; dabbrev.el --- dynamic abbreviation package
 
-;; Copyright (C) 1985, 86, 92, 94, 96, 1997, 2000, 2001
+;; Copyright (C) 1985, 1986, 1992, 1994, 1996, 1997, 2000, 2001, 2002,
+;;   2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
 ;;   Free Software Foundation, Inc.
 
 ;; Author: Don Morrison
+;;	Lars Lindberg
+;; (according to ack.texi)
 ;; Maintainer: Lars Lindberg <Lars.Lindberg@sypro.cap.se>
 ;; Created: 16 Mars 1992
 ;; Lindberg's last update version: 5.7
@@ -11,10 +14,10 @@
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,9 +25,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -61,8 +62,7 @@
 ;; the article for expansion):
 ;; (set (make-local-variable 'dabbrev-friend-buffer-function)
 ;;      (lambda (buffer)
-;;         (save-excursion
-;;           (set-buffer buffer)
+;;         (with-current-buffer buffer
 ;;           (memq major-mode '(news-reply-mode gnus-article-mode)))))
 
 
@@ -100,24 +100,24 @@
 ;;----------------------------------------------------------------
 
 (defgroup dabbrev nil
-  "Dynamic Abbreviations"
+  "Dynamic Abbreviations."
   :tag "Dynamic Abbreviations"
   :group 'abbrev
   :group 'convenience)
 
 (defcustom dabbrev-backward-only nil
-  "*If non-nil, `dabbrev-expand' only looks backwards."
+  "If non-nil, `dabbrev-expand' only looks backwards."
   :type 'boolean
   :group 'dabbrev)
 
 (defcustom dabbrev-limit nil
-  "*Limits region searched by `dabbrev-expand' to this many chars away."
+  "Limits region searched by `dabbrev-expand' to this many chars away."
   :type '(choice (const :tag "off" nil)
 		 integer)
   :group 'dabbrev)
 
 (defcustom dabbrev-abbrev-skip-leading-regexp nil
-  "*Regexp for skipping leading characters of an abbreviation.
+  "Regexp for skipping leading characters of an abbreviation.
 
 Example: Set this to \"\\\\$\" for programming languages
 in which variable names may appear with or without a leading `$'.
@@ -128,14 +128,14 @@ Set this to nil if no characters should be skipped."
 		 (const :tag "off" nil))
   :group 'dabbrev)
 
-(defcustom dabbrev--eliminate-newlines t
-  "*Non-nil means dabbrev should not insert newlines.
+(defcustom dabbrev-eliminate-newlines t
+  "Non-nil means dabbrev should not insert newlines.
 Instead it converts them to spaces."
   :type 'boolean
   :group 'dabbrev)
 
 (defcustom dabbrev-case-fold-search 'case-fold-search
-  "*Control whether dabbrev searches should ignore case.
+  "Control whether dabbrev searches should ignore case.
 A value of nil means case is significant.
 A value of `case-fold-search' means case is significant
  if `case-fold-search' is nil.
@@ -144,31 +144,51 @@ Any other non-nil version means case is not significant."
 		 (const :tag "like search" case-fold-search)
 		 (other :tag "on" t))
   :group 'dabbrev)
+;;;###autoload(put 'dabbrev-case-fold-search 'risky-local-variable t)
 
 (defcustom dabbrev-upcase-means-case-search nil
-  "*The significance of an uppercase character in an abbreviation.
-nil means case fold search, non-nil means case sensitive search.
+  "The significance of an uppercase character in an abbreviation.
+A nil value means case fold search when searching for possible expansions;
+non-nil means case sensitive search.
 
 This variable has an effect only when the value of
 `dabbrev-case-fold-search' says to ignore case."
   :type 'boolean
   :group 'dabbrev)
 
-(defcustom dabbrev-case-replace 'case-replace
-  "*Controls whether dabbrev preserves case when expanding the abbreviation.
-A value of nil means preserve case.
-A value of `case-replace' means preserve case if `case-replace' is nil.
-Any other non-nil version means do not preserve case.
+(defcustom dabbrev-case-distinction 'case-replace
+  "Whether dabbrev treats expansions as the same if they differ in case.
+
+A value of nil means treat them as different.
+A value of `case-replace' means distinguish them if `case-replace' is nil.
+Any other non-nil value means to treat them as the same.
 
 This variable has an effect only when the value of
 `dabbrev-case-fold-search' specifies to ignore case."
   :type '(choice (const :tag "off" nil)
-		 (const :tag "like M-x query-replace" case-replace)
+		 (const :tag "based on `case-replace'" case-replace)
+		 (other :tag "on" t))
+  :group 'dabbrev
+  :version "22.1")
+
+(defcustom dabbrev-case-replace 'case-replace
+  "Whether dabbrev applies the abbreviations's case pattern to the expansion.
+
+A value of nil means preserve the expansion's case pattern.
+A value of `case-replace' means preserve it if `case-replace' is nil.
+Any other non-nil value means modify the expansion
+by applying the abbreviation's case pattern to it.
+
+This variable has an effect only when the value of
+`dabbrev-case-fold-search' specifies to ignore case."
+  :type '(choice (const :tag "off" nil)
+		 (const :tag "based on `case-replace'" case-replace)
 		 (other :tag "on" t))
   :group 'dabbrev)
+;;;###autoload(put 'dabbrev-case-replace 'risky-local-variable t)
 
 (defcustom dabbrev-abbrev-char-regexp nil
-  "*Regexp to recognize a character in an abbreviation or expansion.
+  "Regexp to recognize a character in an abbreviation or expansion.
 This regexp will be surrounded with \\\\( ... \\\\) when actually used.
 
 Set this variable to \"\\\\sw\" if you want ordinary words or
@@ -193,7 +213,7 @@ The recommended value is \"\\\\sw\\\\|\\\\s_\"."
   :group 'dabbrev)
 
 (defcustom dabbrev-check-all-buffers t
-  "*Non-nil means dabbrev package should search *all* buffers.
+  "Non-nil means dabbrev package should search *all* buffers.
 
 Dabbrev always searches the current buffer first.  Then, if
 `dabbrev-check-other-buffers' says so, it searches the buffers
@@ -206,21 +226,21 @@ or matched by `dabbrev-ignored-regexps'."
   :group 'dabbrev)
 
 (defcustom dabbrev-ignored-buffer-names '("*Messages*" "*Buffer List*")
-  "*List of buffer names that dabbrev should not check.
+  "List of buffer names that dabbrev should not check.
 See also `dabbrev-ignored-buffer-regexps'."
   :type '(repeat (string :tag "Buffer name"))
   :group 'dabbrev
   :version "20.3")
 
 (defcustom dabbrev-ignored-buffer-regexps nil
-  "*List of regexps matching names of buffers that dabbrev should not check.
+  "List of regexps matching names of buffers that dabbrev should not check.
 See also `dabbrev-ignored-buffer-names'."
   :type '(repeat regexp)
   :group 'dabbrev
   :version "21.1")
 
 (defcustom dabbrev-check-other-buffers t
-  "*Should \\[dabbrev-expand] look in other buffers?\
+  "Should \\[dabbrev-expand] look in other buffers?\
 
 nil: Don't look in other buffers.
 t: Also look for expansions in the buffers pointed out by
@@ -247,7 +267,7 @@ for an example.
 A mode setting this variable should make it buffer local.")
 
 (defcustom dabbrev-friend-buffer-function 'dabbrev--same-major-mode-p
-  "*A function to decide whether dabbrev should search OTHER-BUFFER.
+  "A function to decide whether dabbrev should search OTHER-BUFFER.
 The function should take one argument, OTHER-BUFFER, and return
 non-nil if that buffer should be searched.  Have a look at
 `dabbrev--same-major-mode-p' for an example.
@@ -264,7 +284,8 @@ A mode setting this variable should make it buffer local."
   "If non-nil, a list of buffers which dabbrev should search.
 If this variable is non-nil, dabbrev will only look in these buffers.
 It will not even look in the current buffer if it is not a member of
-this list.")
+this list."
+  :group 'dabbrev)
 
 ;;----------------------------------------------------------------
 ;; Internal variables
@@ -313,13 +334,16 @@ this list.")
 ;; The regexp for recognizing a character in an abbreviation.
 (defvar dabbrev--abbrev-char-regexp nil)
 
+;; The progress reporter for buffer-scanning progress.
+(defvar dabbrev--progress-reporter nil)
+
 ;;----------------------------------------------------------------
 ;; Macros
 ;;----------------------------------------------------------------
 
-;;; Get the buffer that mini-buffer was activated from
 (defsubst dabbrev--minibuffer-origin ()
-  (car (cdr (buffer-list))))
+  "Get the buffer from which mini-buffer."
+  (window-buffer (minibuffer-selected-window)))
 
 ;; Make a list of some of the elements of LIST.
 ;; Check each element of LIST, storing it temporarily in the
@@ -339,11 +363,9 @@ this list.")
 ;; Exported functions
 ;;----------------------------------------------------------------
 
-;;;###autoload
-(define-key esc-map "/" 'dabbrev-expand)
-;;;??? Do we want this?
-;;;###autoload
-(define-key esc-map [?\C-/] 'dabbrev-completion)
+;;;###autoload (define-key esc-map "/" 'dabbrev-expand)
+;;??? Do we want this?
+;;;###autoload (define-key esc-map [?\C-/] 'dabbrev-completion)
 
 ;;;###autoload
 (defun dabbrev-completion (&optional arg)
@@ -351,22 +373,20 @@ this list.")
 Like \\[dabbrev-expand] but finds all expansions in the current buffer
 and presents suggestions for completion.
 
-With a prefix argument, it searches all buffers accepted by the
+With a prefix argument ARG, it searches all buffers accepted by the
 function pointed out by `dabbrev-friend-buffer-function' to find the
 completions.
 
-If the prefix argument is 16 (which comes from C-u C-u),
-then it searches *all* buffers.
-
-With no prefix argument, it reuses an old completion list
-if there is a suitable one already."
-
+If the prefix argument is 16 (which comes from \\[universal-argument] \\[universal-argument]),
+then it searches *all* buffers."
   (interactive "*P")
   (dabbrev--reset-global-variables)
   (let* ((dabbrev-check-other-buffers (and arg t))
 	 (dabbrev-check-all-buffers
 	  (and arg (= (prefix-numeric-value arg) 16)))
 	 (abbrev (dabbrev--abbrev-at-point))
+         (beg (progn (search-backward abbrev) (point)))
+         (end (progn (search-forward abbrev) (point)))
 	 (ignore-case-p (and (if (eq dabbrev-case-fold-search 'case-fold-search)
 				 case-fold-search
 			       dabbrev-case-fold-search)
@@ -375,79 +395,42 @@ if there is a suitable one already."
 	 (my-obarray dabbrev--last-obarray)
 	 init)
     (save-excursion
-      (if (and (null arg)
-	       my-obarray
-	       (or (eq dabbrev--last-completion-buffer (current-buffer))
-		   (and (window-minibuffer-p (selected-window))
-			(eq dabbrev--last-completion-buffer
-			    (dabbrev--minibuffer-origin))))
-	       dabbrev--last-abbreviation
-	       (>= (length abbrev) (length dabbrev--last-abbreviation))
-	       (string= dabbrev--last-abbreviation
-			(substring abbrev 0
-				   (length dabbrev--last-abbreviation)))
-	       (setq init (try-completion abbrev my-obarray)))
-	  ;; We can reuse the existing completion list.
-	  nil
-	;;--------------------------------
-	;; New abbreviation to expand.
-	;;--------------------------------
-	(setq dabbrev--last-abbreviation abbrev)
-	;; Find all expansion
-	(let ((completion-list
-	       (dabbrev--find-all-expansions abbrev ignore-case-p))
-	      (completion-ignore-case ignore-case-p))
-	  ;; Make an obarray with all expansions
-	  (setq my-obarray (make-vector (length completion-list) 0))
-	  (or (> (length my-obarray) 0)
-	      (error "No dynamic expansion for \"%s\" found%s"
-		     abbrev
-		     (if dabbrev--check-other-buffers "" " in this-buffer")))
-	  (cond
-	   ((or (not ignore-case-p)
-		(not dabbrev-case-replace))
-	    (mapc (function (lambda (string)
-			      (intern string my-obarray)))
-		    completion-list))
-	   ((string= abbrev (upcase abbrev))
-	    (mapc (function (lambda (string)
-			      (intern (upcase string) my-obarray)))
-		    completion-list))
-	   ((string= (substring abbrev 0 1)
-		     (upcase (substring abbrev 0 1)))
-	    (mapc (function (lambda (string)
-			      (intern (capitalize string) my-obarray)))
-		    completion-list))
-	   (t
-	    (mapc (function (lambda (string)
-			      (intern (downcase string) my-obarray)))
-		    completion-list)))
-	  (setq dabbrev--last-obarray my-obarray)
-	  (setq dabbrev--last-completion-buffer (current-buffer))
-	  ;; Find the longest common string.
-	  (setq init (try-completion abbrev my-obarray)))))
-    ;;--------------------------------
-    ;; Let the user choose between the expansions
-    ;;--------------------------------
-    (or (stringp init)
-	(setq init abbrev))
-    (cond
-     ;; * Replace string fragment with matched common substring completion.
-     ((and (not (string-equal init ""))
-	   (not (string-equal (downcase init) (downcase abbrev))))
-      (if (> (length (all-completions init my-obarray)) 1)
-	  (message "Repeat `%s' to see all completions"
-		   (key-description (this-command-keys)))
-	(message "The only possible completion"))
-      (dabbrev--substitute-expansion nil abbrev init nil))
-     (t
-      ;; * String is a common substring completion already.  Make list.
-      (message "Making completion list...")
-      (with-output-to-temp-buffer " *Completions*"
-	(display-completion-list (all-completions init my-obarray)))
-      (message "Making completion list...done")))
-    (and (window-minibuffer-p (selected-window))
-	 (message nil))))
+      ;;--------------------------------
+      ;; New abbreviation to expand.
+      ;;--------------------------------
+      (setq dabbrev--last-abbreviation abbrev)
+      ;; Find all expansion
+      (let ((completion-list
+	     (dabbrev--find-all-expansions abbrev ignore-case-p))
+	    (completion-ignore-case ignore-case-p))
+	;; Make an obarray with all expansions
+	(setq my-obarray (make-vector (length completion-list) 0))
+	(or (> (length my-obarray) 0)
+	    (error "No dynamic expansion for \"%s\" found%s"
+		   abbrev
+		   (if dabbrev--check-other-buffers "" " in this-buffer")))
+	(cond
+	 ((or (not ignore-case-p)
+	      (not dabbrev-case-replace))
+	  (mapc (function (lambda (string)
+			    (intern string my-obarray)))
+		completion-list))
+	 ((string= abbrev (upcase abbrev))
+	  (mapc (function (lambda (string)
+			    (intern (upcase string) my-obarray)))
+		completion-list))
+	 ((string= (substring abbrev 0 1)
+		   (upcase (substring abbrev 0 1)))
+	  (mapc (function (lambda (string)
+			    (intern (capitalize string) my-obarray)))
+		completion-list))
+	 (t
+	  (mapc (function (lambda (string)
+			    (intern (downcase string) my-obarray)))
+		completion-list)))
+	(setq dabbrev--last-obarray my-obarray)
+	(setq dabbrev--last-completion-buffer (current-buffer))))
+    (completion-in-region beg end my-obarray)))
 
 ;;;###autoload
 (defun dabbrev-expand (arg)
@@ -492,7 +475,7 @@ See also `dabbrev-abbrev-char-regexp' and \\[dabbrev-completion]."
 	    (setq direction dabbrev--last-direction))
 	;; If the user inserts a space after expanding
 	;; and then asks to expand again, always fetch the next word.
-	(if (and (eq (preceding-char) ?\ )
+	(if (and (eq (preceding-char) ?\s)
 		 (markerp dabbrev--last-abbrev-location)
 		 (marker-position dabbrev--last-abbrev-location)
 		 (= (point) (1+ dabbrev--last-abbrev-location)))
@@ -500,25 +483,27 @@ See also `dabbrev-abbrev-char-regexp' and \\[dabbrev-completion]."
 	      ;; The "abbrev" to expand is just the space.
 	      (setq abbrev " ")
 	      (save-excursion
-		(if dabbrev--last-buffer
-		    (set-buffer dabbrev--last-buffer))
-		;; Find the end of the last "expansion" word.
-		(if (or (eq dabbrev--last-direction 1)
-			(and (eq dabbrev--last-direction 0)
-			     (< dabbrev--last-expansion-location (point))))
-		    (setq dabbrev--last-expansion-location
-			  (+ dabbrev--last-expansion-location
-			     (length dabbrev--last-expansion))))
-		(goto-char dabbrev--last-expansion-location)
-		;; Take the following word, with intermediate separators,
-		;; as our expansion this time.
-		(re-search-forward
-		 (concat "\\(\\(" dabbrev--abbrev-char-regexp "\\)+\\)"))
-		(setq expansion (buffer-substring-no-properties
-				 dabbrev--last-expansion-location (point)))
+		(save-restriction
+		  (widen)
+		  (if dabbrev--last-buffer
+		      (set-buffer dabbrev--last-buffer))
+		  ;; Find the end of the last "expansion" word.
+		  (if (or (eq dabbrev--last-direction 1)
+			  (and (eq dabbrev--last-direction 0)
+			       (< dabbrev--last-expansion-location (point))))
+		      (setq dabbrev--last-expansion-location
+			    (+ dabbrev--last-expansion-location
+			       (length dabbrev--last-expansion))))
+		  (goto-char dabbrev--last-expansion-location)
+		  ;; Take the following word, with intermediate separators,
+		  ;; as our expansion this time.
+		  (re-search-forward
+		   (concat "\\(?:" dabbrev--abbrev-char-regexp "\\)+"))
+		  (setq expansion (buffer-substring-no-properties
+				   dabbrev--last-expansion-location (point)))
 
-		;; Record the end of this expansion, in case we repeat this.
-		(setq dabbrev--last-expansion-location (point)))
+		  ;; Record the end of this expansion, in case we repeat this.
+		  (setq dabbrev--last-expansion-location (point))))
 	      ;; Indicate that dabbrev--last-expansion-location is
 	      ;; at the end of the expansion.
 	      (setq dabbrev--last-direction -1))
@@ -583,32 +568,32 @@ See also `dabbrev-abbrev-char-regexp' and \\[dabbrev-completion]."
 ;; Local functions
 ;;----------------------------------------------------------------
 
-;;; Checks if OTHER-BUFFER has the same major mode as current buffer.
 (defun dabbrev--same-major-mode-p (other-buffer)
+  "Check if OTHER-BUFFER has the same major mode as current buffer."
   (eq major-mode
-      (save-excursion
-	(set-buffer other-buffer)
+      (with-current-buffer other-buffer
 	major-mode)))
 
-;;; Back over all abbrev type characters and then moves forward over
-;;; all skip characters.
 (defun dabbrev--goto-start-of-abbrev ()
+  "Back over all abbrev type characters and then moves forward over
+all skip characters."
   ;; Move backwards over abbrev chars
   (save-match-data
-    (if (not (bobp))
-	(progn
-	  (forward-char -1)
-	  (while (and (looking-at dabbrev--abbrev-char-regexp)
-		      (not (bobp)))
-	    (forward-char -1))
-	  (or (looking-at dabbrev--abbrev-char-regexp)
-	      (forward-char 1))))
+    (when (> (point) (minibuffer-prompt-end))
+      (forward-char -1)
+      (while (and (looking-at dabbrev--abbrev-char-regexp)
+		  (> (point) (minibuffer-prompt-end))
+		  (not (= (point) (field-beginning (point) nil
+						   (1- (point))))))
+	(forward-char -1))
+      (or (looking-at dabbrev--abbrev-char-regexp)
+	  (forward-char 1)))
     (and dabbrev-abbrev-skip-leading-regexp
 	 (while (looking-at dabbrev-abbrev-skip-leading-regexp)
 	   (forward-char 1)))))
 
-;;; Extract the symbol at point to serve as abbreviation.
 (defun dabbrev--abbrev-at-point ()
+  "Extract the symbol at point to serve as abbreviation."
   ;; Check for error
   (if (bobp)
       (error "No possible abbreviation preceding point"))
@@ -623,10 +608,8 @@ See also `dabbrev-abbrev-char-regexp' and \\[dabbrev-completion]."
     (save-match-data
       (if (save-excursion
 	    (forward-char -1)
-	    (not (looking-at (concat "\\("
-				     (or dabbrev-abbrev-char-regexp
-					 "\\sw\\|\\s_")
-				     "\\)+"))))
+	    (not (looking-at (or dabbrev-abbrev-char-regexp
+                                 "\\sw\\|\\s_"))))
 	  (if (re-search-backward (or dabbrev-abbrev-char-regexp
 				      "\\sw\\|\\s_")
 				  nil t)
@@ -637,8 +620,8 @@ See also `dabbrev-abbrev-char-regexp' and \\[dabbrev-completion]."
     (buffer-substring-no-properties
      dabbrev--last-abbrev-location (point))))
 
-;;; Initializes all global variables
 (defun dabbrev--reset-global-variables ()
+  "Initialize all global variables."
   ;; dabbrev--last-obarray and dabbrev--last-completion-buffer
   ;; must not be reset here.
   (setq dabbrev--last-table nil
@@ -687,7 +670,11 @@ of the expansion in `dabbrev--last-expansion-location'."
 	  (while (and (> count 0)
 		      (setq expansion (dabbrev--search abbrev
 						       reverse
-						       ignore-case)))
+						       (and ignore-case
+							    (if (eq dabbrev-case-distinction 'case-replace)
+								case-replace
+							      dabbrev-case-distinction))
+						       )))
 	    (setq count (1- count))))
 	(and expansion
 	     (setq dabbrev--last-expansion-location (point)))
@@ -704,10 +691,6 @@ If IGNORE-CASE is non-nil, accept matches which differ in case."
 	(setq all-expansions (cons expansion all-expansions))))
     all-expansions))
 
-(defun dabbrev--scanning-message ()
-  (unless (window-minibuffer-p (selected-window))
-    (message "Scanning `%s'" (buffer-name (current-buffer)))))
-
 (defun dabbrev--ignore-buffer-p (buffer)
   "Return non-nil if BUFFER should be ignored by dabbrev."
   (let ((bn (buffer-name buffer)))
@@ -727,14 +710,13 @@ DIRECTION = 0 means try both backward and forward.
 IGNORE-CASE non-nil means ignore case when searching.
 This sets `dabbrev--last-direction' to 1 or -1 according
 to the direction in which the occurrence was actually found.
-It sets `dabbrev--last-expansion-location' to the location 
+It sets `dabbrev--last-expansion-location' to the location
 of the start of the occurrence."
   (save-excursion
     ;; If we were scanning something other than the current buffer,
     ;; continue scanning there.
     (when dabbrev--last-buffer
-      (set-buffer dabbrev--last-buffer)
-      (dabbrev--scanning-message))
+      (set-buffer dabbrev--last-buffer))
     (or
      ;; ------------------------------------------
      ;; Look backward in current buffer.
@@ -766,18 +748,20 @@ of the start of the occurrence."
 	 ;; If we have just now begun to search other buffers,
 	 ;; determine which other buffers we should check.
 	 ;; Put that list in dabbrev--friend-buffer-list.
-	 (or dabbrev--friend-buffer-list
-	     (setq dabbrev--friend-buffer-list
-		   (dabbrev--make-friend-buffer-list))))
+	 (unless dabbrev--friend-buffer-list
+           (setq dabbrev--friend-buffer-list
+                 (dabbrev--make-friend-buffer-list))
+           (setq dabbrev--progress-reporter
+                 (make-progress-reporter
+                  "Scanning for dabbrevs..."
+                  (- (length dabbrev--friend-buffer-list)) 0 0 1 1.5))))
        ;; Walk through the buffers till we find a match.
        (let (expansion)
 	 (while (and (not expansion) dabbrev--friend-buffer-list)
-	   (setq dabbrev--last-buffer
-		 (car dabbrev--friend-buffer-list))
-	   (setq dabbrev--friend-buffer-list
-		 (cdr dabbrev--friend-buffer-list))
+	   (setq dabbrev--last-buffer (pop dabbrev--friend-buffer-list))
 	   (set-buffer dabbrev--last-buffer)
-	   (dabbrev--scanning-message)
+           (progress-reporter-update dabbrev--progress-reporter
+                                     (- (length dabbrev--friend-buffer-list)))
 	   (setq dabbrev--last-expansion-location (point-min))
 	   (setq expansion (dabbrev--try-find abbrev nil 1 ignore-case)))
 	 expansion)))))
@@ -830,7 +814,8 @@ of the start of the occurrence."
 
 (defun dabbrev--safe-replace-match (string &optional fixedcase literal)
   (if (eq major-mode 'picture-mode)
-      (picture-replace-match string fixedcase literal)
+      (with-no-warnings
+       (picture-replace-match string fixedcase literal))
     (replace-match string fixedcase literal)))
 
 ;;;----------------------------------------------------------------
@@ -869,23 +854,28 @@ to record whether we upcased the expansion, downcased it, or did neither."
     ;; matches the start of the expansion,
     ;; copy the expansion's case
     ;; instead of downcasing all the rest.
-    ;; Treat a one-capital-letter abbrev as "not all upper case",
-    ;; so as to force preservation of the expansion's pattern
-    ;; if the expansion starts with a capital letter.
-    (let ((expansion-rest (substring expansion 1)))
-      (if (and (not (and (or (string= expansion-rest (downcase expansion-rest))
-			     (string= expansion-rest (upcase expansion-rest)))
-			 (or (string= abbrev (downcase abbrev))
-			     (and (string= abbrev (upcase abbrev))
-				  (> (length abbrev) 1)))))
-	       (string= abbrev
-			(substring expansion 0 (length abbrev))))
+    ;;
+    ;; Treat a one-capital-letter (possibly with preceding non-letter
+    ;; characters) abbrev as "not all upper case", so as to force
+    ;; preservation of the expansion's pattern if the expansion starts
+    ;; with a capital letter.
+    (let ((expansion-rest (substring expansion 1))
+	  (first-letter-position (string-match "[[:alpha:]]" abbrev)))
+      (if (or (null first-letter-position)
+	      (and (not (and (or (string= expansion-rest (downcase expansion-rest))
+				 (string= expansion-rest (upcase expansion-rest)))
+			     (or (string= abbrev (downcase abbrev))
+				 (and (string= abbrev (upcase abbrev))
+				      (> (- (length abbrev) first-letter-position)
+					 1)))))
+		   (string= abbrev
+			    (substring expansion 0 (length abbrev)))))
 	  (setq use-case-replace nil)))
 
     ;; If the abbrev and the expansion are both all-lower-case
     ;; then don't do any conversion.  The conversion would be a no-op
     ;; for this replacement, but it would carry forward to subsequent words.
-    ;; The goal of this is to preven that carrying forward.
+    ;; The goal of this is to prevent that carrying forward.
     (if (and (string= expansion (downcase expansion))
 	     (string= abbrev (downcase abbrev)))
 	(setq use-case-replace nil))
@@ -897,15 +887,21 @@ to record whether we upcased the expansion, downcased it, or did neither."
     ;; record if we upcased or downcased the first word,
     ;; in order to do likewise for subsequent words.
     (and record-case-pattern
-	 (setq dabbrev--last-case-pattern 
+	 (setq dabbrev--last-case-pattern
 	       (and use-case-replace
 		    (cond ((equal abbrev (upcase abbrev)) 'upcase)
 			  ((equal abbrev (downcase abbrev)) 'downcase)))))
 
-    ;; Convert newlines to spaces.
-    (if dabbrev--eliminate-newlines
-	(while (string-match "\n" expansion)
-	  (setq expansion (replace-match " " nil nil expansion))))
+    ;; Convert whitespace to single spaces.
+    (if dabbrev-eliminate-newlines
+	(let ((pos
+	       (if (equal abbrev " ") 0 (length abbrev))))
+	  ;; If ABBREV is real, search after the end of it.
+	  ;; If ABBREV is space and we are copying successive words,
+	  ;; search starting at the front.
+	  (while (string-match "[\n \t]+" expansion pos)
+	    (setq pos (1+ (match-beginning 0)))
+	    (setq expansion (replace-match " " nil nil expansion)))))
 
     (if old
 	(save-excursion
@@ -948,7 +944,10 @@ Leaves point at the location of the start of the expansion."
 			    "\\(" dabbrev--abbrev-char-regexp "\\)"))
 	  (pattern2 (concat (regexp-quote abbrev)
 			   "\\(\\(" dabbrev--abbrev-char-regexp "\\)+\\)"))
-	  (found-string nil))
+	  ;; This makes it possible to find matches in minibuffer prompts
+	  ;; even when they are "inviolable".
+	  (inhibit-point-motion-hooks t)
+	  found-string result)
       ;; Limited search.
       (save-restriction
 	(and dabbrev-limit
@@ -971,8 +970,8 @@ Leaves point at the location of the start of the expansion."
 	      nil
 	    ;; We have a truly valid match.  Find the end.
 	    (re-search-forward pattern2)
-	    (setq found-string (buffer-substring-no-properties
-				(match-beginning 1) (match-end 1)))
+	    (setq found-string (match-string-no-properties 0))
+	    (setq result found-string)
 	    (and ignore-case (setq found-string (downcase found-string)))
 	    ;; Ignore this match if it's already in the table.
 	    (if (dabbrev-filter-elements
@@ -980,26 +979,21 @@ Leaves point at the location of the start of the expansion."
 		 (string= found-string table-string))
 		(setq found-string nil)))
 	  ;; Prepare to continue searching.
-	  (if reverse
-	      (goto-char (match-beginning 0))
-	    (goto-char (match-end 0))))
+	  (goto-char (if reverse (match-beginning 0) (match-end 0))))
 	;; If we found something, use it.
-	(if found-string
-	    ;; Put it into `dabbrev--last-table'
-	    ;; and return it (either downcased, or as is).
-	    (let ((result (buffer-substring-no-properties
-			   (match-beginning 0) (match-end 0))))
-	      (setq dabbrev--last-table
-		    (cons found-string dabbrev--last-table))
-	      (if (and ignore-case (eval dabbrev-case-replace))
-		  result
-		result)))))))
+	(when found-string
+	  ;; Put it into `dabbrev--last-table'
+	  ;; and return it (either downcased, or as is).
+	  (setq dabbrev--last-table
+		(cons found-string dabbrev--last-table))
+	  result)))))
 
-(dolist (mess '("^No dynamic expansion for .* found$"
+(dolist (mess '("^No dynamic expansion for .* found"
 		"^No further dynamic expansion for .* found$"
 		"^No possible abbreviation preceding point$"))
   (add-to-list 'debug-ignored-errors mess))
 
 (provide 'dabbrev)
 
+;; arch-tag: 29e58596-f080-4306-a409-70296cf9d46f
 ;;; dabbrev.el ends here
