@@ -1,6 +1,6 @@
 /* Add entries to the GNU Emacs Program Manager folder.
    Copyright (C) 1995, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
-     2008, 2009  Free Software Foundation, Inc.
+     2008, 2009, 2010, 2011  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
