@@ -1,6 +1,6 @@
 /* xfont.c -- X core font driver.
-   Copyright (C) 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
-   Copyright (C) 2006, 2007, 2008, 2009
+   Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011
      National Institute of Advanced Industrial Science and Technology (AIST)
      Registration Number H13PRO009
 
@@ -22,6 +22,7 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <setjmp.h>
 #include <X11/Xlib.h>
 
 #include "lisp.h"
@@ -150,7 +151,9 @@ struct font_driver xfont_driver =
     xfont_text_extents,
     xfont_draw,
     NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
-    xfont_check
+    xfont_check,
+    NULL, /* get_variation_glyphs */
+    NULL, /* filter_properties */
   };
 
 extern Lisp_Object QCname;
@@ -489,7 +492,7 @@ xfont_list_pattern (Display *display, char *pattern,
   x_uncatch_errors ();
   UNBLOCK_INPUT;
 
-  font_add_log ("xfont-list", build_string (pattern), list);
+  FONT_ADD_LOG ("xfont-list", build_string (pattern), list);
   return list;
 }
 
@@ -628,7 +631,7 @@ xfont_match (frame, spec)
     }
   UNBLOCK_INPUT;
 
-  font_add_log ("xfont-match", spec, entity);
+  FONT_ADD_LOG ("xfont-match", spec, entity);
   return entity;
 }
 
@@ -714,7 +717,7 @@ xfont_open (f, entity, pixel_size)
   registry = AREF (entity, FONT_REGISTRY_INDEX);
   if (font_registry_charsets (registry, &encoding, &repertory) < 0)
     {
-      font_add_log ("  x:unknown registry", registry, Qnil);
+      FONT_ADD_LOG ("  x:unknown registry", registry, Qnil);
       return Qnil;
     }
 
@@ -730,7 +733,7 @@ xfont_open (f, entity, pixel_size)
   len = font_unparse_xlfd (entity, pixel_size, name, 512);
   if (len <= 0 || (len = xfont_encode_coding_xlfd (name)) < 0)
     {
-      font_add_log ("  x:unparse failed", entity, Qnil);
+      FONT_ADD_LOG ("  x:unparse failed", entity, Qnil);
       return Qnil;
     }
 
@@ -761,7 +764,7 @@ xfont_open (f, entity, pixel_size)
       len = font_unparse_xlfd (temp, pixel_size, name, 512);
       if (len <= 0 || (len = xfont_encode_coding_xlfd (name)) < 0)
 	{
-	  font_add_log ("  x:unparse failed", temp, Qnil);
+	  FONT_ADD_LOG ("  x:unparse failed", temp, Qnil);
 	  return Qnil;
 	}
       xfont = XLoadQueryFont (display, name);
@@ -805,7 +808,7 @@ xfont_open (f, entity, pixel_size)
 
   if (! xfont)
     {
-      font_add_log ("  x:open failed", build_string (name), Qnil);
+      FONT_ADD_LOG ("  x:open failed", build_string (name), Qnil);
       return Qnil;
     }
 
@@ -858,7 +861,7 @@ xfont_open (f, entity, pixel_size)
 
       val = Ffont_get (font_object, QCavgwidth);
       if (INTEGERP (val))
-	font->average_width = XINT (val);
+	font->average_width = XINT (val) / 10;
       if (font->average_width < 0)
 	font->average_width = - font->average_width;
       if (font->average_width == 0
